// File: Multimap.java
// Example of a map with multiple values for each key, and each value is a pair
// Lists all words in any file, and number of occurrences,
//      storing the words by letter

import java.util.*;
import java.io.*;

public class MultimapMap {

  // Read the input file name from the user
  public static String GetFileName() throws java.io.IOException
  { DataInputStream FileNameInput=new DataInputStream(System.in);
    System.out.print("File to Open >");
    System.out.flush();
    return(FileNameInput.readLine());
  }

    public static void main(String[] args) throws java.io.IOException
    {   // Associate the disk file with the File object
        File OutputFile=new File(GetFileName());
        // Now, open a stream from the input file
        PrintWriter DataOutput=
                        new PrintWriter(OutputFile);
        // Read words from file and put into a simulated multimap
        Map<Character, Hashtable<String,Integer>> WordLists
	    = new TreeMap<Character, Hashtable<String,Integer>>();
        try {
	    Scanner s = new Scanner(new File(args[0]));
            while (s.hasNext()) {
		String word = s.next();
		char firstLet=Character.toUpperCase(word.charAt(0));
		if (firstLet>='A' && firstLet <= 'Z') {
                  // get the map for this word, if any
                  Hashtable<String,Integer> Tab=WordLists.get(firstLet);
                  // No entry in WordLists for first letter
                  if (Tab==null) {
                    // Create the table for this letter
                    Tab=new Hashtable<String,Integer>();
                    // Put the word in with a multiplicity of 1
                    Tab.put(word,1);
                    // Put the entry into the map
                    WordLists.put(firstLet,Tab);
                  }
                  else {  // The letter is in the map already
                    // Is the word in the hash table?
                    Integer Count=Tab.get(word);
                    if (Count==null)    // if not, put it in with count of 1
                      Tab.put(word,1);
                    else                // if so, increment its count
                      Tab.put(word,Count+1);
                  }
                }  // if firstLet  
            }
        } catch (IOException e) {
            System.err.println(e);
            System.exit(1);
        }

        // Print all lists
        // First, iterate over all values
        System.out.println("Just the lists....");
        for (Hashtable<String,Integer> l : WordLists.values())
          System.out.println(l);
        // Now, get the lists by key; note use of char in for loop and get
        System.out.println("\n**********Lists prefaced by their key**********");
        for (char c : WordLists.keySet())
          System.out.println(c + ": " + WordLists.get(c));
        DataOutput.println(WordLists);
        DataOutput.close();
    }
}
